@@ -228,6 +228,7 @@ class pyxtal:
         diag = False,
         t_factor = 1.0,
         max_count = 10,
+        torsions = None,
         force_pass = False,
     ):
         if self.molecular:
@@ -245,8 +246,8 @@ class pyxtal:
                 if dim == 3:
                     if numIons is None:
                         numIons = [len(Group(group)[0])]*len(species)
-                    struc = molecular_crystal(group, species, numIons, factor,
-                    lattice=lattice, sites=sites, conventional=conventional, diag=diag, tm=tm)
+                    struc = molecular_crystal(group, species, numIons, factor, lattice=lattice, 
+                    torsions=torsions, sites=sites, conventional=conventional, diag=diag, tm=tm)
                 elif dim == 2:
                     struc = molecular_crystal_2D(group, species, numIons, factor,
                     thickness=thickness, sites=sites, conventional=conventional, tm=tm)
@@ -299,18 +300,26 @@ class pyxtal:
                 pass
 
 
-    def from_seed(self, seed, molecules=None, tol=1e-4, a_tol=5.0, relax_h=False, backend='pymatgen'):
+    def from_seed(self, seed, molecules=None, tol=1e-4, a_tol=5.0, relax_H=False, ignore_HH=True, backend='pymatgen'):
         """
         Load the seed structure from Pymatgen/ASE/POSCAR/CIFs
         Internally they will be handled by Pymatgen
+
+        Args:
+            seed: cif/poscar file or a Pymatgen Structure object
+            molecules: a list of reference molecule (xyz file or Pyxtal molecule)
+            tol: scale factor for covalent bond distance
+            relax_H: whether or not relax the position for hydrogen atoms in structure
+            ignore_HH: whether or not ignore the short H-H distance when checking molecule
+        
         """
 
         if self.molecular:
             pmols = []
             for mol in molecules:
-                pmols.append(pyxtal_molecule(mol)) #.mol
-
-            struc = structure_from_ext(seed, pmols, relax_h=relax_h)
+                pmols.append(pyxtal_molecule(mol, fix=True)) #.mol
+            #QZ: the default choice will not work for molecular H2, which should be rare!
+            struc = structure_from_ext(seed, pmols, relax_H=relax_H, ignore_HH=ignore_HH)
             self.mol_sites = struc.make_mol_sites()
             self.group = Group(struc.wyc.number)
             self.lattice = struc.lattice
@@ -318,6 +327,7 @@ class pyxtal:
             self.numMols = struc.numMols
             self.diag = struc.diag
             self.valid = True # Need to add a check function
+            self.optimize_lattice()
         else:
             if isinstance(seed, dict):
                 self.from_dict()
@@ -577,7 +587,7 @@ class pyxtal:
             return new_strucs
 
 
-    def subgroup_once(self, eps=0.1, H=None, permutations=None, group_type='t', max_cell=4, mut_lat=True):
+    def subgroup_once(self, eps=0.1, H=None, permutations=None, group_type='t', max_cell=4, mut_lat=True, ignore_special=False):
         """
         generate a structure with lower symmetry (for atomic crystals only)
 
@@ -626,8 +636,11 @@ class pyxtal:
                                     if len(ops) < len(splitter.H[0]):
                                         special = True
                                         break
-                        if not special:
+                        if ignore_special:
                             return self._subgroup_by_splitter(splitter, eps=eps, mut_lat=mut_lat)
+                        else:
+                            if not special:
+                                return self._subgroup_by_splitter(splitter, eps=eps, mut_lat=mut_lat)
                     else:
                         #print("try to find the next subgroup")
                         trail_struc = self._subgroup_by_splitter(splitter, eps=eps, mut_lat=mut_lat)
@@ -670,17 +683,21 @@ class pyxtal:
         """
         generate the subgroup dictionary
         """
+        #transform from p21/n to p21/n
+        if self.diag:
+            self.transform([[1,0,0],[0,1,0],[1,0,1]])
+
         t_types = []
         k_types = []
         if group_type == 't':
-            dicts = self.group.get_max_t_subgroup()#['subgroup']
+            dicts = self.group.get_max_t_subgroup()
             t_types = ['t']*len(dicts['subgroup'])
         elif group_type == 'k':
-            dicts = self.group.get_max_k_subgroup()#['subgroup']
+            dicts = self.group.get_max_k_subgroup()
             k_types = ['k']*len(dicts['subgroup'])
         else:
-            dicts = self.group.get_max_t_subgroup()#['subgroup']
-            dict2 = self.group.get_max_k_subgroup()#['subgroup']
+            dicts = self.group.get_max_t_subgroup()
+            dict2 = self.group.get_max_k_subgroup()
             t_types = ['t']*len(dicts['subgroup'])
             k_types = ['k']*len(dict2['subgroup'])
             for key in dicts.keys():
@@ -698,15 +715,13 @@ class pyxtal:
             else:
                 # for molecular crystals, assume the cell does not change
                 for i, tran in enumerate(trans):
+                    tran = np.abs(tran[:3,:3])
                     good = True
-                    # QZ: This loop needs a generalization!
-                    if abs(np.linalg.det(tran[:3,:3])-1)>1e-3:
-                        good = False
-                    elif self.group.number in [5, 7, 8, 9, 12, 13, 14, 15] and self.diag and Hs[i]==4:
-                        good = False
-                    elif self.group.number in [31] and Hs[i]==7:
+                    # only accepts trans like [a, b, c] [b, c, a]
+                    if abs(abs(np.linalg.det(tran))-1)>1e-3: 
                         good = False
                     if good:
+                        #print(np.linalg.det(tran), tran)
                         idx.append(i)
         else:
             for id in idx:
@@ -737,11 +752,13 @@ class pyxtal:
             eps (float): maximum atomic displacement in Angstrom
             mut_lat (bool): whether or not mutate the lattice
         """
+        #print(splitter)
         lat1 = np.dot(splitter.R[:3,:3].T, self.lattice.matrix)
         multiples = np.linalg.det(splitter.R[:3,:3])
         new_struc = self.copy()
         new_struc.group = splitter.H
         lattice = Lattice.from_matrix(lat1, ltype=new_struc.group.lattice_type)
+        #print(lattice); print(lattice.matrix)
         if mut_lat:
             lattice=lattice.mutate(degree=eps, frozen=True)
 
@@ -749,35 +766,43 @@ class pyxtal:
         split_sites = []
         if self.molecular:
             # below only works when the cell does not change
+            # Fix when a, b, c swaps
             for i, site in enumerate(self.mol_sites):
                 pos = site.position
                 mol = site.molecule
                 ori = site.orientation
-                coord0 = mol.mol.cart_coords.dot(ori.matrix.T)
-                coord0 = np.dot(coord0, splitter.R[:3,:3])
+                coord0 = np.dot(mol.mol.cart_coords, ori.matrix.T)
 
                 wp1 = site.wp
                 ori.reset_matrix(np.eye(3))
                 id = 0
-                for ops1, ops2 in zip(splitter.G2_orbits[i], splitter.H_orbits[i]):
-                    #reset molecule
-                    rot = wp1.generators_m[id].affine_matrix[:3,:3].T
-                    #print(rot)
-                    coord1 = np.dot(coord0, rot)
-                    _mol = mol.copy()
-                    _mol.reset_positions(coord1)
-                    #if id != 0: _mol.apply_inversion()
+                for g1s, ops1, ops2 in zip(splitter.G1_orbits[i], splitter.G2_orbits[i], splitter.H_orbits[i]):
+                    if site.wp.multiplicity == len(self.group[0]):
+                        #general wyc
+                        rot = g1s[0].affine_matrix[:3,:3].T
+                    else:
+                        #for special wyc, needs to get better treatment
+                        rot = wp1.generators_m[id].affine_matrix[:3,:3].T
 
-                    pos0 = apply_ops(pos, ops1)[0]
+                    # xyz in new lattice
+                    #coord1 = np.dot(coord0, rot)
+                    #coord1 = np.dot(coord1, splitter.inv_R[:3,:3].T)
+                    #coord1 = np.array([np.dot(splitter.R[:3,:3].T, coord) for coord in coord1])
+                    frac = np.dot(np.dot(coord0, self.lattice.inv_matrix), rot)
+                    frac = np.dot(frac, splitter.inv_R[:3,:3].T)
+                    coord1 = np.dot(frac, lattice.matrix)
+
+                    _mol = mol.copy()
+                    center = _mol.get_center(coord1)
+                    _mol.reset_positions(coord1-center)
+                    #print('============'); print(_mol.mol.to(fmt='xyz'))
+                    pos0 = apply_ops(pos, ops1)[0] #; print(pos0, pos)
                     pos0 -= np.floor(pos0)
                     dis = (np.random.sample(3) - 0.5).dot(self.lattice.matrix)
                     dis /= np.linalg.norm(dis)
                     pos0 += eps*dis*(np.random.random()-0.5)
                     wp, _ = Wyckoff_position.from_symops(ops2, h, permutation=False)
-                    if h in [5, 7, 8, 9, 12, 13, 14, 15] and self.group.number == 31:
-                        diag = True
-                    else:
-                        diag = self.diag
+                    diag = self.diag
                     split_sites.append(mol_site(_mol, pos0, ori, wp, lattice, diag))
                     id += wp.multiplicity
             new_struc.mol_sites = split_sites
@@ -870,6 +895,7 @@ class pyxtal:
         if self.molecular:
             numspecies = self.numMols
             species = [str(mol) for mol in self.molecules]
+            #print(species, numspecies)
         else:
             specie_list = []
             for site in self.atom_sites:
@@ -885,6 +911,25 @@ class pyxtal:
             formula += "{:s}{:d}".format(s, int(i))
         self.formula = formula
 
+    def get_zprime(self, integer=False):
+        """
+        get zprime for molecular xtal
+        """
+        mult = len(self.group[0])
+        comp = [c/mult for c in self.numMols]
+        if integer:
+            comp = [int(c) for c in comp]
+        return comp
+
+    def get_num_torsions(self):
+        """
+        get number of torsions for molecular xtal
+        """
+        N_torsions = 0
+        for s in self.mol_sites:
+            N_torsions += len(s.molecule.torsionlist)
+        return N_torsions
+
     def to_ase(self, resort=True):
         """
         export to ase Atoms object.
@@ -970,6 +1015,13 @@ class pyxtal:
             else:
                 break
 
+    def get_std_representation(self, trans):
+        """
+        perform cell transformation so that the symmetry operations 
+        follow standard space group notation
+        """
+        pass
+
     def transform(self, trans, lattice=None):
         """
         perform cell transformation which may lead to the change of symmetry operation
@@ -988,6 +1040,7 @@ class pyxtal:
         else:
             sites = self.atom_sites
 
+        change_lat = True
         for j, site in enumerate(sites):
             #print("old lattice"); print(site.lattice); print(site.lattice.matrix)
             pos_abs = np.dot(site.position, self.lattice.matrix)
@@ -1004,40 +1057,66 @@ class pyxtal:
                     op1 = op.from_rotation_and_translation(op.rotation_matrix, vec)
                     ops[k] = op1
                 wp, perm = Wyckoff_position.from_symops(ops, self.group.number)
-
                 #print('perm', perm)
-                if not isinstance(perm, list):
-                    diag = True
-                else:
-                    diag = False
-                    # maybe p21/a
-                    pos_frac = pos_frac[perm]
-                
-                # from p21/n to p21/c
-                #if self.diag and not diag:
-                
-                if self.molecular:
-                    ori = site.orientation
-                    #print("new lattice"); print(site.lattice); print(site.lattice.matrix)
-                    if site.diag and not diag:
-                        if perm == [2, 1, 0]:
-                            #print("from p21/n to p21/a to p21/c")
-                            lattice0 = lattice.swap_axis(ids=perm)
-                            xyz, _ = site._get_coords_and_species(absolute=True, first=True)
-                            xyz = np.dot(xyz, lattice.inv_matrix) #frac
-                            xyz = np.dot(xyz[:, perm], lattice0.matrix)
-                            #center = pos_frac.dot(lattice0.matrix)
+                if wp is not None: #QZ: needs to debug
+                    if not isinstance(perm, list):
+                        diag = True
+                    else:
+                        diag = False
+                        # maybe p21/a
+                        pos_frac = pos_frac[perm]
+                    
+                    # from p21/n to p21/c
+                    #if self.diag and not diag:
+                    
+                    if self.molecular:
+                        ori = site.orientation
+                        #print("new lattice"); print(site.lattice); print(site.lattice.matrix)
+                        if site.diag and not diag:
+                            if perm == [2, 1, 0]:
+                                #print("from p21/n to p21/a to p21/c")
+                                lattice0 = lattice.swap_axis(ids=perm)
+                                xyz, _ = site._get_coords_and_species(absolute=True, first=True)
+                                xyz = np.dot(xyz, lattice.inv_matrix) #frac
+                                xyz = np.dot(xyz[:, perm], lattice0.matrix)
+                                #center = pos_frac.dot(lattice0.matrix)
+                                center = site.molecule.get_center(xyz)
+                                site.molecule.reset_positions(xyz-center)
+                                ori.reset_matrix(np.eye(3))
+
+                            elif not np.allclose(lattice.matrix, np.triu(lattice.matrix)):
+                                #print("from p21/n to p21/c")
+                                site.lattice = lattice
+                                xyz, _ = site._get_coords_and_species(absolute=False, first=True)
+                                lattice0 = Lattice.from_matrix(lattice.matrix, 
+                                                               ltype=lattice.ltype, 
+                                                               reset=True)
+
+                                xyz = np.dot(xyz, lattice0.matrix)
+                                center = site.molecule.get_center(xyz)
+                                site.molecule.reset_positions(xyz-center)
+                                ori.reset_matrix(np.eye(3))
+                            else:
+                                lattice0 = lattice
+                        elif diag and not site.diag:
+                            #print("from p21/c to p21/n")
+                            site.lattice = lattice
+                            xyz, _ = site._get_coords_and_species(absolute=False, first=True)
+                            #print(self.lattice); print(self.lattice.matrix)
+                            lattice0 = Lattice.from_matrix(lattice.matrix, 
+                                                      ltype=lattice.ltype, 
+                                                      reset=True)
+                            xyz = np.dot(xyz, lattice0.matrix)
                             center = site.molecule.get_center(xyz)
+                            site.lattice = lattice0
                             site.molecule.reset_positions(xyz-center)
                             ori.reset_matrix(np.eye(3))
-
                         elif not np.allclose(lattice.matrix, np.triu(lattice.matrix)):
-                            #print("from p21/n to p21/c")
                             site.lattice = lattice
                             xyz, _ = site._get_coords_and_species(absolute=False, first=True)
                             lattice0 = Lattice.from_matrix(lattice.matrix, 
-                                                           ltype=lattice.ltype, 
-                                                           reset=True)
+                                                      ltype=lattice.ltype, 
+                                                      reset=True)
 
                             xyz = np.dot(xyz, lattice0.matrix)
                             center = site.molecule.get_center(xyz)
@@ -1045,39 +1124,19 @@ class pyxtal:
                             ori.reset_matrix(np.eye(3))
                         else:
                             lattice0 = lattice
-                    elif diag and not site.diag:
-                        #print("from p21/c to p21/n")
-                        site.lattice = lattice
-                        xyz, _ = site._get_coords_and_species(absolute=False, first=True)
-                        #print(self.lattice); print(self.lattice.matrix)
-                        lattice0 = Lattice.from_matrix(lattice.matrix, 
-                                                  ltype=lattice.ltype, 
-                                                  reset=True)
-                        xyz = np.dot(xyz, lattice0.matrix)
-                        center = site.molecule.get_center(xyz)
-                        site.lattice = lattice0
-                        site.molecule.reset_positions(xyz-center)
-                        ori.reset_matrix(np.eye(3))
-                    elif not np.allclose(lattice.matrix, np.triu(lattice.matrix)):
-                        site.lattice = lattice
-                        xyz, _ = site._get_coords_and_species(absolute=False, first=True)
-                        lattice0 = Lattice.from_matrix(lattice.matrix, 
-                                                  ltype=lattice.ltype, 
-                                                  reset=True)
-
-                        xyz = np.dot(xyz, lattice0.matrix)
-                        center = site.molecule.get_center(xyz)
-                        site.molecule.reset_positions(xyz-center)
-                        ori.reset_matrix(np.eye(3))
+                        #print("reset lattice"); print(lattice); print(lattice.matrix)
+                        #print("reset lattice"); print(lattice0); print(lattice0.matrix)
+                        #print(self.group.symbol, wp)
+                        mol = site.molecule
+                        sites[j] = mol_site(mol, pos_frac, ori, wp, lattice0, diag)
                     else:
-                        lattice0 = lattice
-                    #print("reset lattice"); print(lattice); print(lattice.matrix)
-                    #print("reset lattice"); print(lattice0); print(lattice0.matrix)
-                    #print(self.group.symbol, wp)
-                    mol = site.molecule
-                    sites[j] = mol_site(mol, pos_frac, ori, wp, lattice0, diag)
+                        sites[j] = atom_site(wp, pos_frac, site.specie, diag)
                 else:
-                    sites[j] = atom_site(wp, pos_frac, site.specie, diag)
+                    change_lat = False
+                    #print(wp)
+                    #print("BUG to 1.cif")
+                    #self.to_file('1.cif')
+                    #import sys; sys.exit()
             else:
                 diag = False
                 #print(self.group.symbol)
@@ -1100,37 +1159,13 @@ class pyxtal:
                 sites[j].position = pos_frac
                 sites[j].lattice = lattice0
 
-        if self.molecular:
-            self.lattice = lattice0
-        else:
-            self.lattice = lattice
-        self.diag = diag
+        if change_lat:
+            if self.molecular:
+                self.lattice = lattice0
+            else:
+                self.lattice = lattice
+            self.diag = diag
 
-#    def save(self, filename=None):
-#        """
-#        Save the structure
-#        Args:
-#            filename: the file to save txt information
-#        """
-#        dict0 = self.save_dict(db_filename)
-#        with open(filename, "w") as fp:
-#            json.dump(dict0, fp)
-#
-#        print("save the GP model to", filename, "and database to", db_filename)
-#
-#    def load(self, filename=None):
-#        """
-#        load the structure
-#        Args:
-#            filename: the file to save txt information
-#        """
-#        #print(filename)
-#        with open(filename, "r") as fp:
-#            dict0 = json.load(fp)
-#        self.load_from_dict(dict0, N_max=N_max, device=device)
-#        self.fit(opt=opt)
-#        print("load the GP model from ", filename)
-#
     def save_dict(self):
         """
         save the model as a dictionary
@@ -1261,4 +1296,41 @@ class pyxtal:
                     return False
             return True
 
+    def get_density(self):
+        """
+        compute density
+        """
+
+        return self.to_pymatgen().density
+
+    def has_special_site(self):
+        """
+        Check if the molecular crystal has a special site
+        """
+        special = False
+        for msite in self.mol_sites:
+            if msite.wp.index > 0:
+                special = True
+                break
+        return special
+
+    def to_subgroup(self):
+        """
+        Tranform a molecular crystal with speical sites to subgroup
+        represenatation with general sites
+        """
+        if self.diag:
+            self.transform([[1,0,0],[0,1,0],[1,0,1]])
+
+        #QZ: below is a tentative solution
+        if self.group.number == 64 and s.mol_sites[0].wp.multiplicity==4:
+            s = self.subgroup_once(eps=0, mut_lat=False, H=61, group_type='k', ignore_special=True)
+        else:
+            s = deepcopy(self)
+
+        sub = s.subgroup_once(eps=0, mut_lat=False)
+        sub.optimize_lattice()
+        sub.source = "subgroup"
+        return sub
+
 
