@@ -1,7 +1,6 @@
 """
 This module handles reading and write crystal files.
 """
-from pyxtal.constants import deg, logo
 import numpy as np
 from pymatgen.core.structure import Structure, Molecule
 from pymatgen.core.bonds import CovalentBond
@@ -10,6 +9,13 @@ from pyxtal.wyckoff_site import atom_site, mol_site, WP_merge
 from pyxtal.molecule import pyxtal_molecule, Orientation, compare_mol_connectivity
 from pyxtal.symmetry import Wyckoff_position, Group
 from pyxtal.lattice import Lattice
+from pyxtal.util import get_symmetrized_pmg
+from pyxtal.constants import deg, logo
+from pyxtal.msg import ReadSeedError
+from pkg_resources import resource_filename
+from monty.serialization import loadfn
+
+bonds = loadfn(resource_filename("pyxtal", "database/bonds.json"))
 
 def write_cif(struc, filename=None, header="", permission='w', sym_num=None, style='mp'):
     """
@@ -70,6 +76,11 @@ def write_cif(struc, filename=None, header="", permission='w', sym_num=None, sty
     lines += '_cell_angle_alpha     {:12.6f}\n'.format(alpha)
     lines += '_cell_angle_beta      {:12.6f}\n'.format(beta)
     lines += '_cell_angle_gamma     {:12.6f}\n'.format(gamma)
+    lines += '_cell_volume          {:12.6f}\n'.format(struc.lattice.volume)
+    #if struc.molecular:
+    #    lines += '_cell_formula_units_Z     {:d}\n'.format(sum(struc.numMols))
+    #else:
+    #    lines += '_cell_formula_units_Z     {:d}\n'.format(sum(struc.numIons))
 
     lines += '\nloop_\n'
     lines += ' _symmetry_equiv_pos_site_id\n'
@@ -188,7 +199,7 @@ def read_cif(filename):
 
 class structure_from_ext():
     
-    def __init__(self, struc, ref_mols, tol=0.2, relax_h=False):
+    def __init__(self, struc, ref_mols, tol=0.2, relax_H=False, ignore_HH=False):
 
         """
         extract the mol_site information from the give cif file 
@@ -198,13 +209,14 @@ class structure_from_ext():
             struc: cif/poscar file or a Pymatgen Structure object
             ref_mols: a list of reference molecule (xyz file or Pyxtal molecule)
             tol: scale factor for covalent bond distance
-            relax_h: whether or not relax the position for hydrogen atoms in structure
+            relax_H: whether or not relax the position for hydrogen atoms in structure
+            ignore_HH: whether or not ignore the short H-H distance when checking molecule
         
         """
 
         for ref_mol in ref_mols:
             if isinstance(ref_mol, str):
-                ref_mol = pyxtal_molecule(ref_mol)
+                ref_mol = pyxtal_molecule(ref_mol, fix=True)
             elif isinstance(ref_mol, pyxtal_molecule):
                 ref_mol = ref_mol
             else:
@@ -222,34 +234,19 @@ class structure_from_ext():
         self.ref_mols = ref_mols
         self.tol = tol
         self.diag = False
-        self.relax_h = relax_h
+        self.relax_H = relax_H
 
-        sga = SpacegroupAnalyzer(pmg_struc)
-        ops = sga.get_space_group_operations()
-        self.wyc, perm = Wyckoff_position.from_symops(ops, sga.get_space_group_number())
-        if self.wyc is not None:
-            self.group = Group(self.wyc.number)
-            if isinstance(perm, list):
-                # Standard setting
-                if perm != [0,1,2]:
-                    lattice = Lattice.from_matrix(pmg_struc.lattice.matrix, ltype=self.group.lattice_type)
-                    latt = lattice.swap_axis(ids=perm, random=False).get_matrix()
-                    coor = pmg_struc.frac_coords[:, perm]
-                    pmg_struc = Structure(latt, pmg_struc.atomic_numbers, coor)
-            else:
-                # Nonstandard setting
-                self.diag = True
-                self.perm = perm
-                self.wyc.diagonalize_symops()
+        sym_struc, number = get_symmetrized_pmg(pmg_struc)
+        group = Group(number)
+        self.group = group
+        self.wyc = group[0]
+        self.perm = [0,1,2]
 
-            molecules = search_molecules_in_crystal(pmg_struc, self.tol)
-            if self.relax_h: molecules = self.addh(molecules)
-            self.pmg_struc = pmg_struc
-            self.lattice = Lattice.from_matrix(pmg_struc.lattice.matrix, ltype=self.group.lattice_type)
-            self.resort(molecules)
-            self.numMols = [len(self.wyc)]
-        else:
-            raise ValueError("Cannot find the space group matching the symmetry operation")
+        molecules = search_molecules_in_crystal(sym_struc, self.tol, ignore_HH=ignore_HH)
+        if self.relax_H: molecules = self.addh(molecules)
+        self.pmg_struc = sym_struc
+        self.lattice = Lattice.from_matrix(sym_struc.lattice.matrix, ltype=group.lattice_type)
+        self.resort(molecules)
 
     def resort(self, molecules):
         from pyxtal.operations import apply_ops, find_ids
@@ -257,6 +254,7 @@ class structure_from_ext():
         # filter out the molecular generators
         lat = self.pmg_struc.lattice.matrix
         inv_lat = self.pmg_struc.lattice.inv_matrix
+        new_lat = self.lattice.matrix
         positions = np.zeros([len(molecules),3])
         for i in range(len(molecules)):
             positions[i, :] = np.dot(molecules[i].cart_coords.mean(axis=0), inv_lat) 
@@ -272,53 +270,65 @@ class structure_from_ext():
                 tmp_ids = find_ids(centers, positions)
                 visited_ids.extend(tmp_ids)
                 mults.append(len(tmp_ids))
-                #print("check", id, tmp_ids)
+                #print("check", id, tmp_ids, mults)
+
         # add position and molecule
-        #print("ids", ids, mults)
+        # print("ids", ids, mults)
+        N_sites = len(ids)
         self.numMols = [0] * len(self.ref_mols)
         self.positions = []
         self.p_mols = []
         self.wps = []
-        for i in range(len(ids)):
-            mol1 = molecules[ids[i]]
-            matched = False
-            for j, mol2 in enumerate(self.ref_mols):
-                match, mapping = compare_mol_connectivity(mol2.mol, mol1)
-                if match:
-                    self.numMols[j] += mults[i]
-                    #rearrange the order
-                    order = [mapping[at] for at in range(len(mol1))]
-                    xyz = mol1.cart_coords[order] 
-                    frac = np.dot(xyz, inv_lat) 
-                    xyz = np.dot(frac, lat) 
-                    # create p_mol
-                    p_mol = mol2.copy() 
-                    center = p_mol.get_center(xyz) 
-                    p_mol.reset_positions(xyz-center)
+        ids_done = []
+        for j, mol2 in enumerate(self.ref_mols):
+            for i, id in enumerate(ids):
+                mol1 = molecules[id]
+                if id not in ids_done and len(mol2.mol) == len(mol1):
+                    match, mapping = compare_mol_connectivity(mol2.mol, mol1)
+                    if match:
+                        #print(self.numMols)
+                        self.numMols[j] += mults[i]
+                        #print(j, mults[i], self.numMols)
+                        # rearrange the order
+                        order = [mapping[at] for at in range(len(mol1))]
+                        xyz = mol1.cart_coords[order] 
+                        frac = np.dot(xyz, inv_lat) 
+                        xyz = np.dot(frac, new_lat) 
+                        # create p_mol
+                        p_mol = mol2.copy() 
+                        center = p_mol.get_center(xyz) 
+                        #print(xyz-center)
+                        p_mol.reset_positions(xyz-center)
 
-                    position = np.dot(center, inv_lat)
-                    position -= np.floor(position)
-                    #print(len(self.pmg_struc), len(self.molecule), len(self.wyc))
-                    # check if molecule is on the special wyckoff position
-                    if mults[i] < len(self.wyc):
-                        #Transform it to the conventional representation
-                        if self.diag: position = np.dot(self.perm, position).T
-                        #print("molecule is on the special wyckoff position")
-                        position, wp, _ = WP_merge(position, self.lattice.matrix, self.wyc, 0.1)
-                        self.wps.append(wp)
-                        #print("After Merge:---"); print(position); print(wp)
-                    else:
-                        self.wps.append(self.wyc)
+                        position = np.dot(center, np.linalg.inv(new_lat))
+                        position -= np.floor(position)
+                        #print(position)
+                        #print(lat)
+                        #print(p_mol.mol.cart_coords[:10] + np.dot(position, new_lat))
+                        # print(len(self.pmg_struc), len(self.molecule), len(self.wyc))
 
-                    self.positions.append(position)
-                    self.p_mols.append(p_mol)
-                    matched = True
-                    break
+                        # check if molecule is on the special wyckoff position
+                        if mults[i] < len(self.wyc):
+                            #Transform it to the conventional representation
+                            if self.diag: position = np.dot(self.perm, position).T
+                            #print("molecule is on the special wyckoff position")
+                            position, wp, _ = WP_merge(position, new_lat, self.wyc, 0.1)
+                            self.wps.append(wp)
+                            self.numMols[j] = len(wp)
+                            #print("After Merge:---"); print(position); print(wp)
+                        else:
+                            self.wps.append(self.wyc)
 
+                        self.positions.append(position)
+                        self.p_mols.append(p_mol)
+                        ids_done.append(id)
 
-            if not matched:
-                print(mol1.to('xyz'))
-                raise RuntimeError("molecule cannot be matched")
+        if len(ids_done) < len(ids):
+            for id in ids:
+                if id not in ids_done:
+                    msg = "This molecule cannot be matched to the reference molecule\n"
+                    msg += molecules[id].to('xyz')
+                    raise ReadSeedError(msg)
 
     def addh(self, molecules):
         """
@@ -340,6 +350,11 @@ class structure_from_ext():
         sites = []
         for mol, pos, wp in zip(self.p_mols, self.positions, self.wps):
             site = mol_site(mol, pos, ori, wp, self.lattice, self.diag)
+            #print(pos)
+            #print(self.lattice.matrix)
+            #print([a.value for a in site.molecule.mol.species])
+            #print(site.molecule.mol.cart_coords)
+            #print(site._get_coords_and_species(absolute=True)[0][:10])
             sites.append(site)
         return sites
 
@@ -377,7 +392,7 @@ class structure_from_ext():
             return display_molecules([self.ref_mol, self.molecule])
 
 
-def search_molecules_in_crystal(struc, tol=0.2, once=False):
+def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
     """
     Function to perform to find the molecule in a Pymatgen structure
 
@@ -385,6 +400,7 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False):
         struc: Pymatgen Structure
         tol: tolerance value to check the connectivity
         once: search only one molecule or all molecules
+        ignore_HH: whether or not ignore the short H-H distance when checking molecule
 
     Returns:
         molecules: list of pymatgen molecules
@@ -394,11 +410,12 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False):
         new_members = []
         for site0 in sites0:
             sites_add, visited = check_one_site(struc, site0, visited)
+            #print(sites_add)
             new_members.extend(sites_add)
         return new_members, visited
     
-    def check_one_site(struc, site0, visited):
-        neigh_sites = struc.get_neighbors(site0, 3.0)
+    def check_one_site(struc, site0, visited, rmax=2.9):
+        neigh_sites = struc.get_neighbors(site0, rmax)
         ids = [m.index for m in visited]
         sites_add = []
         ids_add = []
@@ -406,8 +423,23 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False):
         for site1 in neigh_sites:
             if (site1.index not in ids+ids_add):
                 if CovalentBond.is_bonded(site0, site1, tol):
-                    sites_add.append(site1)
-                    ids_add.append(site1.index)
+                    (d, image) = site0.distance_and_image(site1)
+                    #QZ: use our own bond distance lib
+                    key = "{:s}-{:s}".format(site1.specie.value, site0.specie.value)
+
+                    #sometime the H-H short distance is not avoidable
+                    if key == 'H-H': 
+                        if not ignore_HH:
+                            site1.frac_coords += image
+                            sites_add.append(site1)
+                            ids_add.append(site1.index)
+                    else:
+                        if d < bonds[key]:
+                            site1.frac_coords += image
+                            sites_add.append(site1)
+                            ids_add.append(site1.index)
+                    #else:
+                    #    print(key, d, bonds[key])
         if len(sites_add) > 0:
             visited.extend(sites_add)
 
@@ -436,6 +468,7 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False):
             numbers = [s.specie.number for s in visited]
             molecules.append(Molecule(numbers, coords))
             visited_ids.extend([s.index for s in visited])
+            #print(molecules[-1].to(fmt='xyz')); import sys; sys.exit()
         if once and len(molecules) == 1:
             break
 
