@@ -19,7 +19,7 @@ class Lattice:
         ltype: a string representing the type of lattice (from the above list)
         volume: the volume, in Angstroms cubed, of the lattice
         PBC: A periodic boundary condition list, where 1 means periodic,
-            0 means not periodic. 
+            0 means not periodic.
             Ex: [1, 1, 1] -> full 3d periodicity, [0, 0, 1] -> periodicity
             along the z axis
         kwargs: various values which may be defined. If none are defined,
@@ -141,8 +141,8 @@ class Lattice:
             self.dof = 4
         elif self.ltype in ['orthorhombic', 'Orthorhombic']:
             self.dof = 3
-        elif self.ltype in ['tetragonal', 'Tetragonal', 
-            'hexagonal', 'trigonal', 'Hexagonal', 'Trigonal']:        
+        elif self.ltype in ['tetragonal', 'Tetragonal',
+            'hexagonal', 'trigonal', 'Hexagonal', 'Trigonal']:
             self.dof = 2
         else:
             self.dof = 1
@@ -205,11 +205,32 @@ class Lattice:
             cell = np.dot(tran, self.matrix)
             if id > 0:
                 opt = True
-            #print(cell); import sys; sys.exit()
+            #print("cell:"); print(cell) #; import sys; sys.exit()
             return Lattice.from_matrix(cell, ltype=self.ltype, reset=False), tran, opt
         else:
             return self, np.eye(3), opt
 
+    def transform(self, trans_mat=np.eye(3), reset=False):
+        """
+        Optimize the lattice's inclination angles
+        """
+        if type(trans_mat) == list:
+            trans_mat = np.array(trans_mat)
+        cell = np.dot(trans_mat, self.matrix)
+        return Lattice.from_matrix(cell, ltype=self.ltype, reset=reset)
+
+    def encode(self):
+        a, b, c, alpha, beta, gamma = self.get_para(degree=True)
+        if self.ltype in ['cubic', 'Cubic']:
+            return [a]
+        elif self.ltype in ['hexagonal', 'trigonal', 'Hexagonal', 'Trigonal', 'tetragonal', 'Tetragonal']:
+            return [a, c]
+        elif self.ltype in ['orthorhombic', 'Orthorhombic']:
+            return [a, b, c]
+        elif self.ltype in ['monoclinic', 'Monoclinic']:
+            return [a, b, c, beta]
+        else:
+            return [a, b, c, alpha, beta, gamma]
 
     def mutate(self, degree=0.20, frozen=False):
         """
@@ -345,19 +366,20 @@ class Lattice:
         if self.allow_volume_reset is True:
             self.volume = volume
 
-    def transform(self, tran):
-        """
-        Transform the lattice, assuming the symmetry does not change        
-        mostly designed for alternative wyckoff set
-        Args: 
-            tran: 3*3 matrix
-        """
-        # only applied to triclinic/monoclinic/orthorhombic
-        matrix = tran.dot(self.matrix)
-        (a,b,c,alpha,beta,gamma) = matrix2para(matrix)
-        alpha, beta, gamma = alpha*deg, beta*deg, gamma*deg
-        return self.from_para(a, b, c, alpha, beta, gamma, self.ltype)
- 
+    #def transform(self, tran):
+    #    """
+    #    Transform the lattice, assuming the symmetry does not change
+    #    mostly designed for alternative wyckoff set
+
+    #    Args: 
+    #        tran: 3*3 matrix
+    #    """
+    #    # only applied to triclinic/monoclinic/orthorhombic
+    #    matrix = tran.dot(self.matrix)
+    #    (a,b,c,alpha,beta,gamma) = matrix2para(matrix)
+    #    alpha, beta, gamma = alpha*deg, beta*deg, gamma*deg
+    #    return self.from_para(a, b, c, alpha, beta, gamma, self.ltype)
+
     def swap_axis(self, random=False, ids=None):
         """
         For the lattice
@@ -398,12 +420,12 @@ class Lattice:
             return self.from_para(b, c, a, beta, gamma, alpha, self.ltype)
         else:
             return self
-    
+
     def swap_angle(self, random=True, ids=None):
         # only applied to triclinic/monoclinic #/hexagonal
         """
         If the angle is not 90. There will be two equivalent versions
-        e.g., 80 and 100. 
+        e.g., 80 and 100.
         """
         if self.ltype in ["monoclinic", "Monoclinic"]:
             allowed_ids = ["beta", "No"]
@@ -424,25 +446,25 @@ class Lattice:
         alpha, beta, gamma = alpha*deg, beta*deg, gamma*deg
         if ids is None:
             return self
-        elif ids == "alpha": 
+        elif ids == "alpha":
             return self.from_para(a, b, c, 180-alpha, beta, gamma, self.ltype)
-        elif ids == "beta": 
+        elif ids == "beta":
             return self.from_para(a, b, c, alpha, 180-beta, gamma, self.ltype)
-        elif ids == "gamma": 
+        elif ids == "gamma":
             return self.from_para(a, b, c, alpha, beta, 180-gamma, self.ltype)
         else:
             return self
-    
+
     def add_vacuum(self, coor, frac=True, vacuum=15, PBC=[0, 0, 0]):
         """
-        Adds space above and below a 2D or 1D crystal. 
-    
+        Adds space above and below a 2D or 1D crystal.
+
         Args:
             coor: the relative coordinates of the crystal
             vacuum: the amount of space, in Angstroms, to add above and below
             PBC: A periodic boundary condition list,
-                Ex: [1,1,1] -> full 3d periodicity, [0,0,1] -> periodicity along 
-                the z axis    
+                Ex: [1,1,1] -> full 3d periodicity, [0,0,1] -> periodicity
+                along the z axis
 
         Returns:
             The transformed lattice and coordinates after the vacuum is added
@@ -461,18 +483,24 @@ class Lattice:
         if frac:
             coor = np.dot(absolute_coords, np.linalg.inv(matrix))
         else:
-            coor = absolute_coords 
+            coor = absolute_coords
         return matrix, coor
 
 
     def generate_point(self):
-        point = np.random.RandomState().rand(3)
+
+        # point = np.random.RandomState().rand(3)
+        # QZ: it was here because of multiprocess issue
+        # https://github.com/numpy/numpy/issues/9650
+        # now just fix it
+
+        point = np.random.rand(3)
         if self.ltype in ["spherical", "ellipsoidal"]:
             # Choose a point within an octant of the unit sphere
             while point.dot(point) > 1:  # squared
                 point = np.random.random(3)
             # Randomly flip some coordinates
-            for index, x in enumerate(point):
+            for index in range(len(point)):
                 # Scale the point by the max radius
                 if random.uniform(0, 1) < 0.5:
                     point[index] *= -1
@@ -535,7 +563,7 @@ class Lattice:
                     transformed, you should use the default values for random
                     crystal generation
                 random: If False, keeps the stored values for the lattice geometry
-                    even upon applying reset_matrix. To alter the matrix, 
+                    even upon applying reset_matrix. To alter the matrix,
                     use set_matrix() or set_para
                 'unique_axis': the axis ('a', 'b', or 'c') which is not symmetrically
                     equivalent to the other two
@@ -591,7 +619,7 @@ class Lattice:
                 unique_axis: The unique axis for certain symmetry (and especially layer) groups.
                     Because the symmetry operations are not also transformed, you should use the
                     default values for random crystal generation
-                random: If False, keeps the stored values for the lattice geometry even upon applying
+                random: If False, keeps the stored values for the lattice geometry even applying
                     reset_matrix. To alter the matrix, use set_matrix() or set_para
                 'unique_axis': the axis ('a', 'b', or 'c') which is not symmetrically
                     equivalent to the other two
@@ -610,6 +638,8 @@ class Lattice:
             printx("Error: Lattice matrix must be 3x3", priority=1)
             return
         [a, b, c, alpha, beta, gamma] = matrix2para(m)
+
+        # symmetrize the lattice
         if reset:
             if ltype in ['cubic', 'Cubic']:
                 a = b = c = (a+b+c)/3
@@ -627,7 +657,7 @@ class Lattice:
                 alpha = gamma = np.pi/2
             
             # reset matrix according to the symmetry
-            m = para2matrix([a, b, c, alpha, beta, gamma])
+            m = para2matrix([a, b, c, alpha, beta, gamma], format="upper")
         
         # Initialize a Lattice instance
         volume = np.linalg.det(m)
