@@ -15,7 +15,7 @@ from pyxtal.lattice import Lattice, cellsize
 from pyxtal.wyckoff_site import mol_site, WP_merge
 from pyxtal.molecule import pyxtal_molecule, orientation_in_wyckoff_position
 from pyxtal.symmetry import Group, jk_from_i, choose_wyckoff_molecular
-# from pyxtal.operations import angle
+from pyxtal.msg import CompatibilityError
 
 # Define functions
 # ------------------------------
@@ -64,6 +64,7 @@ class molecular_crystal:
         allow_inversion=True,
         orientations=None,
         lattice=None,
+        torsions=None,
         tm=Tol_matrix(prototype="molecular"),
         sites = None,
         conventional = True,
@@ -86,6 +87,7 @@ class molecular_crystal:
             orientations,
             group,
             lattice,
+            torsions,
             sites,
             conventional,
             tm,
@@ -101,6 +103,7 @@ class molecular_crystal:
         orientations,
         group,
         lattice,
+        torsions,
         sites,
         conventional,
         tm,
@@ -112,6 +115,8 @@ class molecular_crystal:
             self.group = group
         else:
             self.group = Group(group, dim=self.dim)
+        if self.diag and self.group.number not in [5, 7, 8, 9, 12, 13, 14, 15]:
+            self.diag = False
         self.number = self.group.number
         """
         The international group number of the crystal:
@@ -147,9 +152,12 @@ class molecular_crystal:
                 printx(msg, priority=1)
                 return
 
+        if torsions is None:
+            torsions = [None]*len(molecules)
+
         self.molecules = []  # A pyxtal_molecule objects,
-        for mol in molecules:
-            self.molecules.append(pyxtal_molecule(mol, self.tol_matrix))
+        for i, mol in enumerate(molecules):
+            self.molecules.append(pyxtal_molecule(mol, torsions=torsions[i], tm=self.tol_matrix))
 
         self.sites = {}
         for i, mol in enumerate(self.molecules):
@@ -339,7 +347,7 @@ class molecular_crystal:
 
             if n == n0:
                 #print("n == n0", n, n0)
-                return False
+                return False, False
             while True:
                 num = np.dot(n, l_mult)
                 dobackwards = False
@@ -356,7 +364,7 @@ class molecular_crystal:
                 # All combinations failed: return False
                 if n == n0 and p >= len(l_mult) - 1:
                     #print("All combinations failed: return False")
-                    return False
+                    return False, False
                 # Too few atoms
                 if num < numIon:
                     # Forwards routine
@@ -381,10 +389,10 @@ class molecular_crystal:
                             break
         # All species passed: return True
         if has_freedom:
-            return True
+            return True, True
         # All species passed, but no degrees of freedom: return 0
         else:
-            return 0
+            return True, False
 
     def __str__(self):
         s = "------Random Molecular Crystal------"
@@ -431,14 +439,14 @@ class molecular_crystal:
         """
 
         # Check the minimum number of degrees of freedom within the Wyckoff positions
-        degrees = self.check_compatible(self.group, self.numMols, self.valid_orientations)
-        if degrees is False:
+        compat, degrees = self.check_compatible(self.group, self.numMols, self.valid_orientations)
+        if not compat:
             self.valid = False
-            #msg = "the space group is incompatible with the number of molecules"
-            #raise ValueError(msg)
-            return
+            msg = "Compoisition " + str(self.numMols) + " is incompatible with symmetry " + str(self.group.number) 
+            raise CompatibilityError(msg)
+
         else:
-            if degrees == 0:
+            if not degrees:
                 self.lattice_attempts = 20
                 self.coord_attempts = 3
                 self.ori_attempts = 1
@@ -709,6 +717,7 @@ class molecular_crystal_2D(molecular_crystal):
             orientations,
             group,
             lattice,
+            None,
             sites,
             conventional,
             tm,
@@ -779,6 +788,7 @@ class molecular_crystal_1D(molecular_crystal):
             orientations,
             group,
             lattice,
+            None,
             sites,
             conventional,
             tm,
