@@ -10,9 +10,10 @@ constraints.
 """
 # Imports
 import os
+from pkg_resources import resource_filename
 from copy import deepcopy
 from operator import itemgetter
-from random import choice
+from random import choice, random
 import numpy as np
 from scipy.spatial.transform import Rotation
 import networkx as nx
@@ -21,6 +22,7 @@ import networkx as nx
 from pymatgen.core.structure import Molecule
 from pymatgen.symmetry.analyzer import PointGroupAnalyzer, generate_full_symmops
 from pymatgen.core.bonds import CovalentBond
+from monty.serialization import loadfn
 
 # PyXtal imports
 from pyxtal.msg import printx
@@ -31,6 +33,7 @@ from pyxtal.database.collection import Collection
 
 # Define functions
 # ------------------------------
+bonds = loadfn(resource_filename("pyxtal", "database/bonds.json"))
 molecule_collection = Collection("molecules")
 
 def cleaner(list_to_clean):
@@ -113,7 +116,13 @@ class pyxtal_molecule:
         tm: tolerance matrix
     """
 
-    def __init__(self, mol=None, symmetrize=True, tm=Tol_matrix(prototype="molecular")):
+    def __init__(self, 
+                 mol=None, 
+                 symmetrize=True, 
+                 fix=False, 
+                 torsions=None, 
+                 tm=Tol_matrix(prototype="molecular")):
+
         mo = None
         self.smile = None
         self.torsionlist = None
@@ -130,7 +139,7 @@ class pyxtal_molecule:
                         raise NameError("{:s} is not a valid path".format(mol))
                 elif tmp[-1] == 'smi':
                     self.smile = tmp[0]
-                    symbols, xyz, self.torsionlist = self.rdkit_mol_init(tmp[0])
+                    symbols, xyz, self.torsionlist = self.rdkit_mol_init(tmp[0], fix, torsions)
                     mo = Molecule(symbols, xyz)
                     symmetrize = False
                 else:
@@ -296,12 +305,18 @@ class pyxtal_molecule:
         return display_molecules([self.mol])
 
 
-    def rdkit_mol_init(self, smile):
+    def rdkit_mol_init(self, smile, fix, torsions):
         """
         initialize the mol xyz and torsion list
+
+        Args:
+            smile: smile string
+            fix: whether or not fix the seed
+            torsionlist: None or list
         """
         from rdkit import Chem
         from rdkit.Chem import AllChem
+        from rdkit.Geometry import Point3D
 
         smarts_torsion="[*]~[!$(*#*)&!D1]-&!@[!$(*#*)&!D1]~[*]"
         mol = Chem.MolFromSmiles(smile)
@@ -313,11 +328,29 @@ class pyxtal_molecule:
         symbols = []
         for id in range(mol.GetNumAtoms()):
             symbols.append(mol.GetAtomWithIdx(id).GetSymbol())
+        if fix or torsions is not None:
+            AllChem.EmbedMultipleConfs(mol, numConfs=1, randomSeed=0xf00d)
+            conf = mol.GetConformer(0)
+            #print(conf.GetPositions())
+        else:
+            AllChem.EmbedMultipleConfs(mol, numConfs=20, maxAttempts=200, useRandomCoords=True, pruneRmsThresh=0.5)
+            N_confs = mol.GetNumConformers()
+            #print(N_confs)
+            conf = mol.GetConformer(choice(range(N_confs)))
 
-        AllChem.EmbedMultipleConfs(mol, numConfs=2, randomSeed=0xf00d)
-        conf = mol.GetConformer(0)
         #print("Init: ", conf.GetPositions())
-        xyz = self.align(conf)
+        if torsions is not None:
+            xyz = self.set_torsion_angles(conf, torsions, torsionlist=torsionlist)
+        else:
+            xyz = conf.GetPositions()
+            angs = self.get_torsion_angles(xyz, torsionlist)
+            angs *= (1+0.15*np.random.uniform(-1., 1., len(angs))) 
+            xyz = self.set_torsion_angles(conf, angs, torsionlist=torsionlist)
+            xyz -= self.get_center(xyz)
+            #if random() > 0.5:
+            #else:
+            #    xyz = self.align(conf)
+        #print(torsions, "align", self.get_torsion_angles(xyz, torsionlist))
         #print("Init: ", xyz[:3])
         return symbols, xyz, torsionlist
 
@@ -334,7 +367,7 @@ class pyxtal_molecule:
         #print("To_dict", mol.GetConformer(0).GetPositions())
         return mol
 
-    def align(self, conf, reflect=False):
+    def align(self, conf, reflect=False, torsionlist=None):
         """
         Align the molecule and return the xyz
         The default CanonicalizeConformer function may also include inversion
@@ -342,14 +375,14 @@ class pyxtal_molecule:
         from rdkit.Chem import rdMolTransforms as rdmt
 
         #rotation
-        #print("align: "); print(conf.GetPositions())
+        #print("check_xyz", conf.GetPositions())
         trans = rdmt.ComputeCanonicalTransform(conf)
-        if np.linalg.det(trans[:3,:3]) < 0:
-            trans[:3,:3] *= -1
+        #QZ: it is not pure rotation
+        #print("ComputeCanonicalTransform", np.linalg.det(trans[:3,:3]))
+
+        if np.linalg.det(trans[:3,:3]) < 0: trans[:3,:3] *= -1
+        if reflect: trans[:3,:3] *= -1
 
-        if reflect:
-            trans[:3,:3] *= -1
-        #print(trans)
         rdmt.TransformConformer(conf, trans)
         
         #print("rot", conf.GetPositions()[:3])
@@ -398,7 +431,7 @@ class pyxtal_molecule:
 
             return rdmt.ComputePrincipalAxesAndMoments(conf1)
 
-    def get_torsion_angles(self, xyz):
+    def get_torsion_angles(self, xyz, torsionlist=None):
         """
         get the torsion angles
         """
@@ -407,23 +440,26 @@ class pyxtal_molecule:
         mol = self.rdkit_mol(self.smile)
         conf = mol.GetConformer(0)
 
-        for i in range(len(self.mol)):
+        if torsionlist is None: torsionlist=self.torsionlist
+
+        for i in range(len(xyz)):
             x,y,z = xyz[i]
             conf.SetAtomPosition(i,Point3D(x,y,z))
 
         angs = []
-        for torsion in self.torsionlist:
+        for torsion in torsionlist:
             (i, j, k, l) = torsion
             angs.append(rdmt.GetDihedralDeg(conf, i, j, k, l))
         return angs
     
-    def set_torsion_angles(self, conf, angles, reflect=False):
+    def set_torsion_angles(self, conf, angles, reflect=False, torsionlist=None):
         """
         reset the torsion angles and update molecular xyz
         """
         from rdkit.Chem import rdMolTransforms as rdmt
 
-        for id, torsion in enumerate(self.torsionlist):
+        if torsionlist is None: torsionlist=self.torsionlist
+        for id, torsion in enumerate(torsionlist):
             (i, j, k, l) = torsion
             rdmt.SetDihedralDeg(conf, i, j, k, l, angles[id])
         
@@ -1122,28 +1158,16 @@ def make_graph(mol, tol=0.2):
     names = {}
     for i, site in enumerate(mol._sites):
         names[i] = site.specie.value
+        if names[i] not in ["C", "H", "O", "N", "S", "P", "Si", "F", "Cl", "Br", "I"]:
+            raise ValueError(name[i]+' is not supported')
 
     for i in range(len(mol)-1):
         site1 = mol.sites[i]
         for j in range(i+1, len(mol)):
             site2 = mol.sites[j]
-            #remove short X-H distances
-            if names[i] == "H" and names[j]=="H":
-                factor = -0.5
-            elif [names[i], names[j]] in [["S","S"], ["S","O"], ["O","S"], ["F","O"], ["O","F"]]:
-                factor = 0.05
-            elif "H" in [names[i], names[j]]:
-                factor = 0.5
-            elif [names[i], names[j]] in [["N", "S"], ["S", "N"]]:
-                factor = 1.25
-            else:
-                factor = 1.0
-            try:
-                if CovalentBond.is_bonded(site1, site2, factor*tol):
-                    G.add_edge(i,j)
-                    #if 'S' in [names[i], names[j]]: print(names[i], names[j], mol.get_distance(i, j))
-            except ValueError:
-                pass
+            key = "{:s}-{:s}".format(names[i], names[j])
+            if site1.distance(site2) < bonds[key]:
+                G.add_edge(i,j)
     nx.set_node_attributes(G, names, 'name')
 
     return G
