@@ -154,7 +154,7 @@ class TestMolecular(unittest.TestCase):
         struc.mol_sites[0].rotate(ax_vector=[1, 0, 0], angle=90)
         pmg_struc = struc.to_pymatgen()
         sga = SpacegroupAnalyzer(pmg_struc)
-        pmg_struc.to("cif", "1.cif")
+        #pmg_struc.to("cif", "1.cif")
         self.assertTrue(sga.get_space_group_symbol() == "Cmc2_1")
         # print(pmg_struc.frac_coords[:3])
 
@@ -405,8 +405,9 @@ class TestSubgroup(unittest.TestCase):
         #struc.subgroup_once(0.01, None, permutation, max_cell=2) 
 
     def test_molecules(self):
-        for name in ["aspirin", "resorcinol", "coumarin", "HAHCOI", \
-                     "WEXBOS", "MERQIM", "LAGNAL", "YICMOP", "LUFHAW", "JAPWIH"]:
+        for name in ["aspirin", "resorcinol", "coumarin", "HAHCOI", "xxvi",\
+                     "WEXBOS", "MERQIM", "LAGNAL", "YICMOP", "LUFHAW", \
+                     "JAPWIH", "AXOSOW01", "PAHYON01"]:
             cif = cif_path + name + ".cif"
             struc = pyxtal(molecular=True)
             struc.from_seed(seed=cif, molecules=[name])
@@ -414,11 +415,10 @@ class TestSubgroup(unittest.TestCase):
             pmg_s1 = Structure.from_file(cif)
             self.assertTrue(sm.StructureMatcher().fit(pmg_struc, pmg_s1))
 
-            if name not in ["aspirin", "resorcinol", "coumarin"]:
-                Cs = struc.subgroup(eps=0, max_cell=1)
-                for C in Cs:
-                    pmg_s2 = C.to_pymatgen()
-                    self.assertTrue(sm.StructureMatcher().fit(pmg_struc, pmg_s2))
+            Cs = struc.subgroup(eps=0, max_cell=1)
+            for C in Cs:
+                pmg_s2 = C.to_pymatgen()
+                self.assertTrue(sm.StructureMatcher().fit(pmg_struc, pmg_s2))
  
     def test_hydrate(self):
         # glycine dihydrate
