@@ -78,7 +78,7 @@ class mol_site:
         s = "{:} @ [{:7.4f} {:7.4f} {:7.4f}]  ".format(formula, *self.position)
         s += "WP: {:2d}{:s}, ".format(self.wp.multiplicity, self.wp.letter)
         s += "Site symmetry {:} ==> Euler: ".format(self.site_symm)
-        s += "{:6.3f} {:6.3f} {:6.3f}".format(*self.angles)
+        s += "{:6.2f} {:6.2f} {:6.2f}".format(*self.angles)
 
         return s
 
@@ -158,6 +158,7 @@ class mol_site:
         dict0["center"] = self.position - np.floor(self.position) #self.molecule.get_center(xyz)
         dict0["number"] = self.wp.number
         dict0["index"] = self.wp.index
+        dict0["PBC"] = self.wp.PBC
         dict0["dim"] = self.wp.dim
         dict0["diag"] = self.diag
         dict0["lattice"] = self.lattice.get_matrix()
@@ -169,7 +170,7 @@ class mol_site:
     def from_1D_dicts(cls, dicts):
         from pyxtal.molecule import pyxtal_molecule, Orientation
 
-        mol = pyxtal_molecule(mol=dicts['smile']+'.smi')
+        mol = pyxtal_molecule(mol=dicts['smile']+'.smi', fix=True)
         rdkit_mol = mol.rdkit_mol(mol.smile)
         conf = rdkit_mol.GetConformer(0)
         #print("try")
@@ -190,7 +191,7 @@ class mol_site:
         #    print('load'); print(xyz[:3])
         #    print("aaaaaaaaaaaaaa"); print(xyz[:3].dot(orientation.matrix.T))
         #    print("matrix"); print(orientation.matrix)
-        wp = Wyckoff_position.from_group_and_index(g, index, dim)
+        wp = Wyckoff_position.from_group_and_index(g, index, dim, dicts["PBC"])
         diag = dicts["diag"]
         lattice = Lattice.from_matrix(dicts["lattice"], ltype=dicts["lattice_type"])
         position = dicts["center"] #np.dot(dicts["center"], lattice.inv_matrix)
@@ -384,7 +385,6 @@ class mol_site:
         If the list does not change, we return the new coordinates
         otherwise, terminate the calculation.
         """
-        from pyxtal.io import search_molecule_in_crystal
         from pyxtal.molecule import compare_mol_connectivity, Orientation
         try:
             from openbabel import pybel, openbabel
